-- AUTO-GENERATED .SQL FILE - Generated by the MySQL versions tool
USE zimbra;
INSERT INTO zimbra.config(name, value, description) VALUES
	('db.version', '118', 'db schema version'),
	('index.version', '2', 'index version'),
	('redolog.version', '1.42', 'redolog version')
;
COMMIT;
